(ns wanijo.schema.view
  (:require [hiccup.form :as hform]
            [ring.util.anti-forgery :refer [anti-forgery-field]]
            [wanijo.framework.view :as view]
            [wanijo.framework.form :as form]
            [wanijo.framework.routing :refer [path]]
            [wanijo.framework.time :refer [prettify-dt]]
            [wanijo.schema.domain :as domain]
            [wanijo.attribute.domain :as attr-domain]))

(def new-form
  {:fields {:schema-name {:label "Name"
                          :required true
                          :spec ::domain/name}}})

(def new-attr-form
  {:fields {:name {:label "Name"
                   :required true
                   :spec ::attr-domain/name}
            :type {:label "Type"
                   :required true
                   :spec ::attr-domain/type
                   :options [["String" "string"]
                             ["Text" "text"]
                             ["Date" "date"]
                             ["File" "file"]]}
            :required {:label "Required"
                       :required false
                       :spec ::attr-domain/required}}})

(defn overview! [req]
  (let [session (:session req)
        uuid (:uuid session)
        schemas (domain/all-created-by! uuid)]
    (view/layout!
     :session session
     :content
     [[:h1 "All schemas"]
      [:table
       [:thead
        [:tr
         [:th "Name"]
         [:th "Created"]]]
       [:tbody
        (for [schema schemas]
          [:tr
           [:td
            [:a {:href (path :schema-show schema)}
             (:name schema)]]
           [:td
            (prettify-dt (:created_at schema))]])]]
      [:h1 "New schema"]
      (hform/form-to [:post (path :schema-new)]
                     (anti-forgery-field)
                     (form/field new-form :schema-name req)
                     (hform/submit-button "Create"))])))

(defn show-schema! [schema attrs req]
  (view/layout!
   :session (:session req)
   :content
   [[:h1 "Schema "
     [:span.schema-title__name (:name schema)]]
    (hform/form-to [:post "/schema/edit"])
    [:h2 "Attributes"]
    [:ul
     (for [attr attrs]
       [:li (:name attr)])]
    [:h3 "New attribute"]
    (hform/form-to [:post (path :attribute-new)]
                   (anti-forgery-field)
                   (hform/hidden-field "schema" (:uuid schema))
                   (form/field new-attr-form :name req)
                   (form/drop-down new-attr-form :type req)
                   (form/check-box new-attr-form :required req)
                   (hform/submit-button "Save"))
    [:h2 "Actions"]
    (hform/form-to {:class "inline"}
                   [:delete (path :schema-delete schema)]
                   (anti-forgery-field)
                   (hform/submit-button "Delete!"))]))
